`default_nettype none

module examples(clk, reset, in1, in2, out, x, y, xn, yn, y1, y2, y3, A, B,
	control, cpu_data, io_data, int_data, do1, do2);
input clk;
input reset;
input [15:0] in1, in2;
output [15:0] out;
output [3:0] x,y,xn,yn;
output [15:0] y1, y2, y3;
output [7:0] A, B;
input [2:0] control;
input [7:0] cpu_data, io_data, int_data;
output [7:0] do1, do2;

ct10 count10(clk, out[3:0]);

assign_nonblock a1(clk, xn, yn);
assign_yesblock a2(clk, x, y);
assign_c1 a3(clk, in1, in2, y2);
assign_c2 a4(clk, in1, in2, y1);

swap s1(clk, reset, 1, 8'h41, 8'h42, A, B); 

movavg m1(clk, in1, y3);

mux1 mu1(do1, control, cpu_data, io_data, int_data);
mux2 mu2(do2, control, cpu_data, io_data, int_data);

endmodule

module swap(input clk, input set, input swap_en, input [7:0] r1, input [7:0] r2, 
			output reg [7:0] reg_A, output reg [7:0] reg_B);

always @(posedge clk or posedge set) begin
	if (set) begin
		reg_A <= r1;
		reg_B <= r2;
	end else begin
		if (swap_en) begin
			reg_A <= reg_B;
			reg_B <= reg_A;
		end
	end
end

endmodule

module ct10(input clk, output [3:0] o);

reg [3:0] count;
assign o = count;

always @(posedge clk) begin
    count <= count + 1;
    if (count == 10) count <= 0;
end

endmodule

module assign_nonblock(input clk, 
	output reg [3:0] xn, 
	output reg [3:0] yn);

always @(posedge clk) begin
	xn <= xn + 1;
	yn <= yn + 1;
end

endmodule

module assign_yesblock(input clk, 
	output reg [3:0] x, 
	output reg [3:0] y); 

always @(posedge clk) begin
	x = x + 1;
	y = y + 1;
end

endmodule

module assign_c1(input clk,
	input [15:0] input_value,
	input [15:0] center_offset,
	output reg[15:0] y2);

always @(posedge clk) begin
	y2 <= 3*((input_value >> 4) + center_offset);
end

endmodule

module assign_c2(input clk,
	input [15:0] input_value,
	input [15:0] center_offset,
	output reg[15:0] y1);

always @(posedge clk) begin
	y1 = input_value >> 4;
	y1 = y1 + center_offset;
	y1 = 3 * y1;
end

endmodule

module movavg(input clk, input [15:0] current, output reg [15:0] avg);

reg [15:0] history [0:3];
reg [3:0] i;
reg [15+4:0] sum;
always @(posedge clk) begin
    for (i = 1; i < 4; i = i + 1) history[i] <= history[i-1];

    history[0] <= current;
end

always @(posedge clk) begin
    sum = 0;
    for (i = 0; i < 4; i = i + 1) sum = sum + history[i];

    avg = sum/4;
end
endmodule

module mux1(output [7:0] data_out, 
			input [2:0] control, 
			input [7:0] cpu_data, 
			input [7:0] io_data, 
			input [7:0] int_data);

assign data_out = control[0] ? cpu_data :
				  control[1] ? io_data :
				  control[2] ? int_data : 8'hzz;
endmodule

module mux2(output reg [7:0] data_out, 
			input [2:0] control, 
			input [7:0] cpu_data, 
			input [7:0] io_data, 
			input [7:0] int_data);

always 
	case (control)
	3'b001: data_out <= cpu_data;
	3'b010: data_out <= io_data;
	3'b100: data_out <= int_data;
	default:data_out <= 8'hzz; 
	endcase

endmodule
